"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class ASTProgram {
    constructor(statements = []) {
        this.Statements = [];
        this.Statements = statements;
    }
    TokenLiteral() {
        if (this.Statements.length > 0) {
            return this.Statements[0].TokenLiteral();
        }
        else {
            return '';
        }
    }
    String() {
        return this.Statements.map(s => s.String()).join('');
    }
}
exports.ASTProgram = ASTProgram;
class LetStatement {
    constructor(token, name, value, index = null) {
        this.Token = token;
        this.Name = name;
        this.Value = value;
        this.Index = index;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `${this.TokenLiteral()} ${this.Name.String()}${this.Index ? this.Index.String() : ''} = ${this.Value.String()};`;
    }
}
exports.LetStatement = LetStatement;
class ImportSpec {
    constructor(token, path, name = null) {
        this.Token = token;
        this.Path = path;
        this.Name = name;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `import("${this.Path.String()}")${this.Name ? ` as ${this.Name.String()}` : ''}`;
    }
}
exports.ImportSpec = ImportSpec;
class ReturnStatement {
    constructor(token, returnValue = null) {
        this.Token = token;
        this.ReturnValue = returnValue;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let out = `${this.TokenLiteral()} = `;
        if (this.ReturnValue !== null)
            out += this.ReturnValue.String();
        out += ';';
        return out;
    }
}
exports.ReturnStatement = ReturnStatement;
class ExpressionStatement {
    constructor(token, expression) {
        this.Token = token;
        this.Expression = expression;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Expression.String();
    }
}
exports.ExpressionStatement = ExpressionStatement;
class Identifier {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Value;
    }
}
exports.Identifier = Identifier;
class Comment {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Value;
    }
}
exports.Comment = Comment;
class IntegerLiteral {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Token.Literal;
    }
}
exports.IntegerLiteral = IntegerLiteral;
class FloatLiteral {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Token.Literal;
    }
}
exports.FloatLiteral = FloatLiteral;
class PrefixExpression {
    constructor(token, operator, right) {
        this.Token = token;
        this.Operator = operator;
        this.Right = right;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `(${this.Operator}${this.Right.String()})`;
    }
}
exports.PrefixExpression = PrefixExpression;
class IncrementExpression {
    constructor(token, name, prefix) {
        this.Token = token;
        this.Name = name;
        this.Prefix = prefix;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Prefix ? `(++${this.Name})` : `(${this.Name}++)`;
    }
}
exports.IncrementExpression = IncrementExpression;
class DecrementExpression {
    constructor(token, name, prefix) {
        this.Token = token;
        this.Name = name;
        this.Prefix = prefix;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Prefix ? `(--${this.Name})` : `(${this.Name}--)`;
    }
}
exports.DecrementExpression = DecrementExpression;
class RangeLiteral {
    constructor(left, operator, right) {
        this.Left = left;
        this.Operator = operator;
        this.Right = right;
    }
    TokenLiteral() {
        return this.Operator;
    }
    String() {
        return `(${this.Left.String()} ${this.Operator} ${this.Right.String()})`;
    }
}
exports.RangeLiteral = RangeLiteral;
class InfixExpression {
    constructor(token, left, operator, right) {
        this.Right = null;
        this.Token = token;
        this.Left = left;
        this.Operator = operator;
        this.Right = right;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `(${this.Left.String()} ${this.Operator} ${this.Right ? this.Right.String() : ''})`;
    }
}
exports.InfixExpression = InfixExpression;
class AstBoolean {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Token.Literal;
    }
}
exports.AstBoolean = AstBoolean;
class IfExpression {
    constructor(token, expression, consequence, alternative = null) {
        this.Token = token;
        this.Condition = expression;
        this.Consequence = consequence;
        this.Alternative = alternative;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let out = `${this.Condition.String()} ${this.Consequence.String()}`;
        if (this.Alternative != null)
            out += `else ${this.Alternative.String()}`;
        return out;
    }
}
exports.IfExpression = IfExpression;
class BlockStatement {
    constructor(token, statements = []) {
        this.Statements = [];
        this.Token = token;
        this.Statements = statements;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Statements.map(s => s.String()).join('');
    }
}
exports.BlockStatement = BlockStatement;
class WhileLiteral {
    constructor(token, expression, body) {
        this.Token = token;
        this.Expression = expression;
        this.Body = body;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `${this.TokenLiteral()}(${this.Expression.String()}) ${this.Body.String()}`;
    }
}
exports.WhileLiteral = WhileLiteral;
class ForLiteral {
    constructor(token, initiate, check, iterate, body) {
        this.Token = token;
        this.Initiate = initiate;
        this.Check = check;
        this.Iterate = iterate;
        this.Body = body;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return `${this.TokenLiteral()}(${this.Initiate.String()};${this.Check.String()};${this.Iterate.String()}) ${this.Body.String()}`;
    }
}
exports.ForLiteral = ForLiteral;
class FunctionLiteral {
    constructor(token, parameters = [], body) {
        this.Parameters = [];
        this.Token = token;
        this.Parameters = parameters;
        this.Body = body;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let params = this.Parameters.map(p => p.String());
        return `${this.TokenLiteral()}(${params.join(', ')}) ${this.Body.String()}`;
    }
}
exports.FunctionLiteral = FunctionLiteral;
class CallExpression {
    constructor(token, func, args = []) {
        this.Token = token;
        this.Function = func;
        this.Arguments = args;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let args = this.Arguments.map(a => a.String());
        return `${this.Function.String()}(${args.join(', ')})`;
    }
}
exports.CallExpression = CallExpression;
class StringLiteral {
    constructor(token, value) {
        this.Token = token;
        this.Value = value;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        return this.Token.Literal;
    }
}
exports.StringLiteral = StringLiteral;
class ArrayLiteral {
    constructor(token, elements = []) {
        this.Elements = [];
        this.Token = token;
        this.Elements = elements;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let out = '';
        let elements = this.Elements.map(e => e.String());
        out += `[${elements.join(', ')}]`;
        return out;
    }
}
exports.ArrayLiteral = ArrayLiteral;
class IndexExpression {
    constructor(token, left, index = null, hasColon = false, rightIndex = null) {
        this.Index = null;
        this.RightIndex = null;
        this.Token = token;
        this.Left = left;
        this.Index = index;
        this.HasColon = hasColon;
        this.RightIndex = rightIndex;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let index = this.Index ? `:${this.Index.String()}` : null;
        let rightIndex = this.RightIndex ? `:${this.RightIndex.String()}` : null;
        return `(${this.Left.String()}[${index}${rightIndex}])`;
    }
}
exports.IndexExpression = IndexExpression;
class HashLiteral {
    constructor(token, pairs) {
        this.Pairs = new Map();
        this.Token = token;
        this.Pairs = pairs;
    }
    TokenLiteral() {
        return this.Token.Literal;
    }
    String() {
        let pairs = [];
        this.Pairs.forEach((v, k) => pairs.push(`${k.String()}:${v.String()}`));
        return `{${pairs.join(', ')}}`;
    }
}
exports.HashLiteral = HashLiteral;
//# sourceMappingURL=ast.js.map