"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const object_1 = require("./object");
const logger_1 = __importDefault(require("./logger"));
const file_loader_1 = __importDefault(require("./file-loader"));
const process_1 = __importDefault(require("process"));
class Environment {
    constructor(logger, fileLoader) {
        this.store = new Map();
        this.outer = null;
        this.Logger = new logger_1.default();
        this.FileLoader = new file_loader_1.default();
        this._files = {};
        this.Logger = logger;
        this.FileLoader = fileLoader;
    }
    GlobalEnv() {
        if (this.outer) {
            if (this.outer.outer)
                return this.outer.GlobalEnv();
            return this.outer;
        }
        return this;
    }
    Get(name) {
        let obj = this.store.get(name);
        if (!obj && this.outer !== null) {
            obj = this.outer.Get(name);
        }
        return obj ? obj : null;
    }
    Set(name, val) {
        this.store.set(name, val);
        return val;
    }
}
exports.default = Environment;
function NewEnvironment() {
    let logger = new logger_1.default();
    let fileLoader = new file_loader_1.default();
    let env = new Environment(logger, fileLoader);
    return env;
}
exports.NewEnvironment = NewEnvironment;
function NewEnclosedEnvironment(outer) {
    let env = new Environment(outer.Logger);
    env.outer = outer;
    return env;
}
exports.NewEnclosedEnvironment = NewEnclosedEnvironment;
function NewNodeEnvironment() {
    let logger = new logger_1.default();
    let fileLoader = new file_loader_1.default();
    let env = new Environment(logger, fileLoader);
    env.Set('__arch', new object_1.OString(process_1.default.arch));
    env.Set('__argv', jsArrayToMonkey(process_1.default.argv));
    env.Set('__env', jsObjectToMonkey(process_1.default.env));
    env.Set('__environment', new object_1.OString('node'));
    env.Set('__node', new object_1.OBoolean(true));
    env.Set('__pid', new object_1.OInteger(process_1.default.pid));
    env.Set('__platform', new object_1.OString(process_1.default.platform));
    env.Set('__version', new object_1.OString(process_1.default.version));
    let versions = process_1.default.versions;
    env.Set('__versions', jsObjectToMonkey(versions));
    return env;
}
exports.NewNodeEnvironment = NewNodeEnvironment;
function NewBrowserEnvironment() {
    let logger = new logger_1.default();
    let fileLoader = new file_loader_1.default();
    let env = new Environment(logger, fileLoader);
    env.Set('__browser', new object_1.OBoolean(true));
    env.Set('__environment', new object_1.OString('browser'));
    let location = window.location;
    env.Set('__location', jsObjectToMonkey({
        hash: location.hash,
        host: location.host,
        hostname: location.hostname,
        href: location.href,
        origin: location.origin,
        pathname: location.pathname,
        port: location.port,
        protocol: location.protocol,
    }));
    let screen = window.screen;
    env.Set('__screen', jsObjectToMonkey({
        availHeight: screen.availHeight,
        availLeft: screen.availLeft,
        availTop: screen.availTop,
        availWidth: screen.availWidth,
        colorDepth: screen.colorDepth,
        height: screen.height,
        pixelDepth: screen.pixelDepth,
        width: screen.width,
    }));
    let navigator = window.navigator;
    env.Set('__navigator', jsObjectToMonkey({
        appCodeName: navigator.appCodeName,
        appName: navigator.appName,
        appVersion: navigator.appVersion,
        cookieEnabled: navigator.cookieEnabled,
        deviceMemory: navigator.deviceMemory,
        doNotTrack: navigator.doNotTrack,
        language: navigator.language,
        maxTouchPoints: navigator.maxTouchPoints,
        onLine: navigator.onLine,
        platform: navigator.platform,
        product: navigator.product,
        productSub: navigator.productSub,
        userAgent: navigator.userAgent,
        vendor: navigator.vendor,
        vendorSub: navigator.vendorSub,
    }));
    let connection = window.navigator.connection;
    env.Set('__connection', jsObjectToMonkey({
        effectiveType: connection.effectiveType,
        rtt: connection.rtt,
        downlink: connection.downlink,
    }));
    return env;
}
exports.NewBrowserEnvironment = NewBrowserEnvironment;
function jsObjectToMonkey(obj) {
    let pairs = new Map();
    for (let key in obj) {
        let value = new object_1.ONull();
        if (typeof obj[key] === 'string') {
            value = new object_1.OString(obj[key]);
        }
        else if (typeof obj[key] === 'number') {
            value = new object_1.OInteger(obj[key]);
        }
        else if (typeof obj[key] === 'boolean') {
            value = new object_1.OBoolean(obj[key]);
        }
        else if (!obj[key]) {
            value = new object_1.ONull();
        }
        let hashKey = new object_1.OString(key);
        let hashed = hashKey.HashKey();
        pairs.set(hashed.Match, new object_1.HashPair(hashKey, value));
    }
    return new object_1.OHash(pairs);
}
function jsArrayToMonkey(arr) {
    return new object_1.OArray(arr.map(s => new object_1.OString(s)));
}
//# sourceMappingURL=environment.js.map