import Token, { TokenTypeName } from '../token/token';
import File from '../token/file';
import Lexer from '../lexer/lexer';
import { AstBoolean, ASTProgram, BlockStatement, CallExpression, Expression, ExpressionStatement, WhileLiteral, ForLiteral, FunctionLiteral, ImportSpec, Identifier, Comment, IfExpression, InfixExpression, IntegerLiteral, FloatLiteral, PrefixExpression, ReturnStatement, Statement, StringLiteral, IndexExpression, IncrementExpression, DecrementExpression, RangeLiteral } from '../ast/ast';
export declare const LOWEST = 1, LOR = 2, LAND = 3, EQUALS = 4, LESSGREATER = 5, SUM = 6, PRODUCT = 7, PREFIX = 8, BITWISE = 9, CALL = 10, INDEX = 11;
export declare const precedences: {
    [index: string]: number;
};
export default class Parser {
    l: Lexer;
    file: File | null;
    errors: string[];
    comments: string[];
    curToken: Token;
    peekToken: Token;
    prefixParseFns: {
        [index: string]: Function;
    };
    infixParseFns: {
        [index: string]: Function;
    };
    constructor(lexer: Lexer);
    Errors(): string[];
    nextToken(): void;
    ParseProgram(): ASTProgram;
    parseStatement(): Statement | null;
    parseLetStatement(): Statement | null;
    parseReturnStatement(): ReturnStatement;
    parseExpressionStatement(): ExpressionStatement;
    parseExpression(precedence: number): any;
    parseIdentifier(): Identifier | IncrementExpression | DecrementExpression;
    parsePreIncrement(): IncrementExpression;
    parsePreDecrement(): DecrementExpression;
    parseComment(): Comment;
    formatError(t: Token, msg: string): string;
    parseIntegerLiteral(): IntegerLiteral | RangeLiteral | null;
    parseFloatLiteral(): FloatLiteral | null;
    parseStringLiteral(): StringLiteral;
    curTokenIs(t: TokenTypeName): boolean;
    peekTokenIs(t: TokenTypeName): boolean;
    expectPeek(t: TokenTypeName): boolean;
    peekError(t: TokenTypeName): void;
    registerPrefix(tokenType: TokenTypeName, fn: Function): void;
    registerInfix(tokenType: TokenTypeName, fn: Function): void;
    noPrefixParseFnError(t: Token): void;
    parsePrefixExpression(): PrefixExpression;
    peekPrecedence(): number;
    curPrecedence(): number;
    parseInfixExpression(left: Expression): InfixExpression;
    parseBoolean(): AstBoolean;
    parseGroupedExpression(): any;
    parseIfExpression(): IfExpression | null;
    parseBlockStatement(): BlockStatement;
    parseImportSpec(): ImportSpec | null;
    parseFunctionLiteral(): FunctionLiteral | null;
    parseWhileLiteral(): WhileLiteral | null;
    parseForLiteral(): ForLiteral | null;
    parseFunctionParameters(): Identifier[];
    parseCallExpression(func: Expression): CallExpression;
    parseArrayLiteral(): Expression;
    parseExpressionList(end: TokenTypeName): Expression[];
    parseIndexExpression(left: Expression): IndexExpression | null;
    parseHashLiteral(): Expression | null;
}
