"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const position_1 = __importStar(require("./position"));
class File {
    constructor(set, name, base, size, lines = [], infos = []) {
        this.base = 0;
        this.size = 0;
        this.lines = [];
        this.infos = [];
        this.set = set;
        this.name = name;
        this.base = base;
        this.size = size;
        this.lines = lines;
        this.infos = infos;
    }
    Name() {
        return this.name;
    }
    Base() {
        return this.base;
    }
    Size() {
        return this.size;
    }
    LineCount() {
        let n = this.lines.length;
        return n;
    }
    AddLine(offset) {
        let i = this.lines.length;
        if ((i == 0 || this.lines[i - 1] < offset) && offset < this.size) {
            this.lines.push(offset);
        }
    }
    MergeLine(line) {
        if (line <= 0) {
            throw new Error('illegal line number (line numbering starts at 1)');
        }
        if (line >= this.lines.length) {
            throw new Error('illegal line number');
        }
        let copy = this.lines.slice();
        for (let i = line; i < this.lines.length; i++) {
            this.lines[i + 1] = copy[i];
        }
        this.lines = this.lines.slice(0, this.lines.length - 1);
    }
    SetLines(lines) {
        let size = this.size;
        for (let i = 0; i < lines.length; i++) {
            let offset = lines[i];
            if ((i > 0 && offset <= lines[i - 1]) || size <= offset) {
                return false;
            }
        }
        this.lines = lines;
        return true;
    }
    SetLinesForContent(content) {
        let lines = [];
        let line = 0;
        for (let [offset, b] of content) {
            if (line >= 0) {
                lines.push(line);
            }
            line = -1;
            if (b == '\n') {
                line = Number(offset) + 1;
            }
        }
        this.lines = lines;
    }
    AddLineInfo(offset, filename, line) {
        let i = this.infos.length;
        if (i == 0 || (this.infos[i - 1].Offset < offset && offset < this.size)) {
            this.infos.push(new lineInfo(offset, filename, line));
        }
    }
    Pos(offset) {
        if (offset > this.size) {
            throw new Error('illegal file offset');
        }
        return new position_1.default(this.base + offset);
    }
    Offset(p) {
        if (p.Offset < this.base || p.Offset > this.base + this.size) {
            throw new Error('illegal Pos value');
        }
        return p.Offset - this.base;
    }
    Line(p) {
        return this.Position(p).Line;
    }
    PositionFor(p, adjusted) {
        if (p != position_1.NoPos) {
            if (p.Offset < this.base || p.Offset > this.base + this.size) {
                throw new Error('illegal Pos value');
            }
            return this.position(p, adjusted);
        }
        return p;
    }
    Position(p) {
        return this.PositionFor(p, true);
    }
    unpack(offset, adjusted) {
        let filename = this.name;
        let line = 0;
        let column = 0;
        let i = searchInts(this.lines, offset);
        if (i >= 0) {
            [line, column] = [i + 1, offset - this.lines[i] + 1];
        }
        if (adjusted && this.infos.length > 0) {
            let i = searchLineInfos(this.infos, offset);
            if (i >= 0) {
                let alt = this.infos[i];
                filename = alt.Filename;
                let i2 = searchInts(this.lines, alt.Offset);
                if (i2 >= 0) {
                    line += alt.Line - i2 - 1;
                }
            }
        }
        return [filename, line, column];
    }
    position(p, adjusted) {
        let offset = Number(p) - this.base;
        p.Offset = offset;
        [p.Filename, p.Line, p.Column] = this.unpack(offset, adjusted);
        return p;
    }
}
exports.default = File;
class lineInfo {
    constructor(offset, filename, line = 0) {
        this.Offset = 0;
        this.Filename = '';
        this.Line = 0;
        this.Offset = offset;
        this.Filename = filename;
        this.Line = line;
    }
    valueOf() {
        return this.Offset;
    }
}
function searchLineInfos(a, x) {
    for (let i = 0; i < a.length; i++) {
        if (a[i].Offset > x)
            return i;
    }
    return 0;
}
function searchInts(a, x) {
    let [i, j] = [0, a.length];
    while (i < j) {
        let h = i + (j - i) / 2;
        if (a[h] <= x) {
            i = h + 1;
        }
        else {
            j = h;
        }
    }
    return i - 1;
}
//# sourceMappingURL=file.js.map